import math
print("Yet Another Waveguide Tool - Version 0.1.0")

class cFlare:	
	def __init__(self, R, ff, step):
		self.R = R # Flare radius in mm
		self.Factor = ff # Part of horn length reserved for the flare
		self.StepSize = step / 1000 # Step Size in mm

# --- User Input
G = 25.4      # Thoat Diameter in mm
B = 120.0 # Beamwidth in degree
fc = 1000.0    # Low Frequency Limit in Hz


Flare = cFlare(100, 1.0/3.0, 1.0)



# --- Write Control_Solver
fileSolution = open("Solving.txt", "w")
fileSolution.write("Control_Solver\n")
fileSolution.write("  f1=500\n")                    # First frequency
fileSolution.write("  f2=16000\n")                  # Last frequency
fileSolution.write("  NumFrequencies=100\n")        # Number of frequencies
fileSolution.write("  Abscissa=log\n")              # Distribution of frequencies
fileSolution.write("  MeshFrequency=16000Hz\n")     # Frequency up to which analysis is valid
fileSolution.write("  Dim=CircSym\n")               # Circular symmetric model
fileSolution.write("  NUC=true\n")                  # NUC can be applied because interface is extruded.
fileSolution.write("\n")

# --- Write Subdomains
fileSolution.write("SubDomain_Properties\n")
fileSolution.write("  SubDomain=1\n")
fileSolution.write("  ElType=Interior\n")
fileSolution.write("\n")
fileSolution.write("SubDomain_Properties\n")
fileSolution.write("  SubDomain=2\n")
fileSolution.write("  ElType=Exterior\n")
fileSolution.write("\n")

# --- Write Infinite Baffle
fileSolution.write("Infinite_Baffle\n")
fileSolution.write("  SubDomain=2\n")
fileSolution.write("  Position=Pos\n")           # Infinite baffle pointing to positive axis
fileSolution.write("\n")

# --- Calculate Conical Horn
K = 25000 # [m * deg * Hz]
W = K / (B * fc)
alpha = B / 2.0 * math.pi / 180.0
D = 0.5 * G / 1000.0 / math.tan(alpha) 
Lc = 0.5 * W / math.tan(alpha) - D

# --- Write Nodes
fileSolution.write("Nodes \"Nodes\"\n")
fileSolution.write("  Scale=1mm\n")
if Flare.R == 0.0:
	print("Conical Horn\n")
	fileSolution.write("  Shift=" + str(-Lc * 1000) + "mm\n")
	#                   Index  <axial> <radial>
	fileSolution.write("  1000 0.0 0.0\n")
	fileSolution.write("  1001 0.0 " + str(G / 2) + "\n")
	fileSolution.write("  1002 " + str(Lc * 1000) + " " + str(W / 2 * 1000) + "\n")
	fileSolution.write("  6001 " + str(Lc * 1000 + 10) + " " + str(W / 2 * 1000) + "\n") # Outer interface radius
	fileSolution.write("  6002 " + str(Lc * 1000 + 10) + " 0.0 \n") # Outer interface
	fileSolution.write("\n")
	nodes = 3
else:
	print("Conical Horn with Rounded Flare\n")
	R = Flare.R / 1000
	Lc = (1.0 - Flare.Factor) * Lc
	W1 = Lc * math.tan(alpha)
	px = Lc - R * math.sin(alpha)
	py = W1 + R * math.cos(alpha)
	L = px + R
	fileSolution.write("  Shift=" + str(-L * 1000) + "mm\n")
	#                   Index  <axial> <radial>
	fileSolution.write("  1000 0.0 0.0\n")
	fileSolution.write("  1001 0.0 " + str(G / 2) + "\n")
	fileSolution.write("  1002 " + str(Lc * 1000) + " " + str(W1 * 1000) + "\n")
	nodes = 3
	z = Lc + Flare.StepSize
	while(z < L):
		y1 = py - math.sqrt(math.pow(R,2.0) - math.pow((z - px),2.0))
		fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(y1 * 1000) + "\n")
		nodes += 1
		z += Flare.StepSize
		if z >= L:
			z = L
			y1 = py - math.sqrt(math.pow(R,2.0) - math.pow((z - px),2.0))
			fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(y1 * 1000) + "\n")
			nodes += 1
			z += Flare.StepSize
	fileSolution.write("  6001 " + str(L * 1000 + 10) + " " + str(py * 1000) + "\n") # Outer interface radius
	fileSolution.write("  6002 " + str(L * 1000 + 10) + " 0.0 \n") # Outer interface
	fileSolution.write("\n")
	
# --- Write Elements
fileSolution.write("Elements \"Horn\"\n")
fileSolution.write("  RefNodes=\"Nodes\"\n")
fileSolution.write("  SubDomain=1\n")
for ii in range(1, nodes - 1):
	fileSolution.write("  " + str(ii+1) + " " + str(1000 + ii) + " " + str(1000 + ii + 1) + "\n")
fileSolution.write("\n")

# --- Write Horn-Driving
fileSolution.write("Elements \"Horn-Driving\"\n")
fileSolution.write("  RefNodes=\"Nodes\"\n")
fileSolution.write("  SubDomain=1\n")
fileSolution.write("  1 1000 1001\n")
fileSolution.write("\n") 

# --- Write Driver
fileSolution.write("Driving \"Driver\"\n")
fileSolution.write("  RefElements=\"Horn-Driving\"\n")
fileSolution.write("  DrvGroup=1001\n")
fileSolution.write("  DirIsAxial=true\n")
fileSolution.write("\n")  
  
# --- Write Interface Definition
fileSolution.write("Elements \"Interface-IB\"\n")
fileSolution.write("  RefNodes=\"Nodes\"\n")
fileSolution.write("  SubDomain=1,2\n")
fileSolution.write("  1    " + str(1000 + nodes - 1) +"  6001\n")
fileSolution.write("  2    6001  6002\n")
fileSolution.write("\n")

fileSolution.close()
